@@ -6,50 +6,53 @@
 /* SYNOPSIS
 /*	#include <bounce.h>
 /*
-/*	int	bounce_append(flags, id, stats, recipient, relay, dsn)
+/*	int	bounce_append(flags, id, stats, recipient, relay, tstats, dsn)
 /*	int	flags;
 /*	const char *id;
 /*	MSG_STATS *stats;
 /*	RECIPIENT *rcpt;
 /*	const char *relay;
+/*	const TLS_STATS *tstats;
 /*	DSN	*dsn;
 /*
-/*	int	bounce_flush(flags, queue, id, encoding, smtputf8, sender,
+/*	int	bounce_flush(flags, queue, id, encoding, sendopts, sender,
 /*				dsn_envid, dsn_ret)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
 /*	const char *encoding;
-/*	int	smtputf8;
+/*	int	sendopts;
 /*	const char *sender;
 /*	const char *dsn_envid;
 /*	int	dsn_ret;
 /*
-/*	int	bounce_flush_verp(flags, queue, id, encoding, smtputf8,
+/*	int	bounce_flush_verp(flags, queue, id, encoding, sendopts,
 /*				sender, dsn_envid, dsn_ret, verp_delims)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
 /*	const char *encoding;
-/*	int	smtputf8;
+/*	int	sendopts;
 /*	const char *sender;
 /*	const char *dsn_envid;
 /*	int	dsn_ret;
 /*	const char *verp_delims;
 /*
-/*	int	bounce_one(flags, queue, id, encoding, smtputf8, sender,
-/*				dsn_envid, ret, stats, recipient, relay, dsn)
+/*	int	bounce_one(flags, queue, id, encoding, sendopts, sender,
+/*				dsn_envid, ret, stats, recipient, relay,
+/*				tstats, dsn)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
 /*	const char *encoding;
-/*	int	smtputf8;
+/*	int	sendopts;
 /*	const char *sender;
 /*	const char *dsn_envid;
 /*	int	dsn_ret;
 /*	MSG_STATS *stats;
 /*	RECIPIENT *rcpt;
 /*	const char *relay;
+/*	const TLS_STATS *tstats;
 /*	DSN	*dsn;
 /*
 /*	void	bounce_client_init(title, maps)
@@ -58,26 +61,29 @@
 /* INTERNAL API
 /*	DSN_FILTER *delivery_status_filter;
 /*
-/*	int	bounce_append_intern(flags, id, stats, recipient, relay, dsn)
+/*	int	bounce_append_intern(flags, id, stats, recipient, relay, tstats, dsn)
 /*	int	flags;
 /*	const char *id;
 /*	MSG_STATS *stats;
 /*	RECIPIENT *rcpt;
 /*	const char *relay;
+/*	const TLS_STATS *tstats;
+/*	DSN	*dsn;
 /*
-/*	int	bounce_one_intern(flags, queue, id, encoding, smtputf8, sender,
-/*				dsn_envid, ret, stats, recipient, relay, dsn)
+/*	int	bounce_one_intern(flags, queue, id, encoding, sendopts, sender,
+/*			dsn_envid, ret, stats, recipient, relay, tstats, dsn)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
 /*	const char *encoding;
-/*	int	smtputf8;
+/*	int	sendopts;
 /*	const char *sender;
 /*	const char *dsn_envid;
 /*	int	dsn_ret;
 /*	MSG_STATS *stats;
 /*	RECIPIENT *rcpt;
 /*	const char *relay;
+/*	const TLS_STATS *tstats;
 /*	DSN	*dsn;
 /* DESCRIPTION
 /*	This module implements the client interface to the message
@@ -151,10 +157,12 @@
 /*	This information is used for syslogging only.
 /* .IP encoding
 /*	The body content encoding: MAIL_ATTR_ENC_{7BIT,8BIT,NONE}.
-/* .IP smtputf8
-/*	The level of SMTPUTF8 support (to be defined).
+/* .IP sendopts
+/*	Sender-requested SMTPUTF8 or RequireTLS support.
 /* .IP sender
 /*	The sender envelope address.
+/* .IP tstats
+/*	TLS per-feature status.
 /* .IP dsn_envid
 /*	Optional DSN envelope ID.
 /* .IP dsn_ret
@@ -186,6 +194,14 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -221,7 +237,7 @@ DSN_FILTER *delivery_status_filter;
 
 int     bounce_append(int flags, const char *id, MSG_STATS *stats,
 		              RECIPIENT *rcpt, const char *relay,
-		              DSN *dsn)
+		              const TLS_STATS *tstats, DSN *dsn)
 {
     DSN     my_dsn = *dsn;
     DSN    *dsn_res;
@@ -241,17 +257,19 @@ int     bounce_append(int flags, const char *id, MSG_STATS *stats,
     if (delivery_status_filter != 0
     && (dsn_res = dsn_filter_lookup(delivery_status_filter, &my_dsn)) != 0) {
 	if (dsn_res->status[0] == '4')
-	    return (defer_append_intern(flags, id, stats, rcpt, relay, dsn_res));
+	    return (defer_append_intern(flags, id, stats, rcpt, relay, tstats,
+					dsn_res));
 	my_dsn = *dsn_res;
     }
-    return (bounce_append_intern(flags, id, stats, rcpt, relay, &my_dsn));
+    return (bounce_append_intern(flags, id, stats, rcpt, relay, tstats,
+				 &my_dsn));
 }
 
 /* bounce_append_intern - append delivery status to per-message bounce log */
 
 int     bounce_append_intern(int flags, const char *id, MSG_STATS *stats,
 			             RECIPIENT *rcpt, const char *relay,
-			             DSN *dsn)
+			             const TLS_STATS *tstats, DSN *dsn)
 {
     DSN     my_dsn = *dsn;
     int     status;
@@ -262,7 +280,7 @@ int     bounce_append_intern(int flags, const char *id, MSG_STATS *stats,
      */
     if (flags & DEL_REQ_FLAG_MTA_VRFY) {
 	my_dsn.action = "undeliverable";
-	status = verify_append(id, stats, rcpt, relay, &my_dsn,
+	status = verify_append(id, stats, rcpt, relay, tstats, &my_dsn,
 			       DEL_RCPT_STAT_BOUNCE);
 	return (status);
     }
@@ -273,7 +291,7 @@ int     bounce_append_intern(int flags, const char *id, MSG_STATS *stats,
      */
     if (flags & DEL_REQ_FLAG_USR_VRFY) {
 	my_dsn.action = "undeliverable";
-	status = trace_append(flags, id, stats, rcpt, relay, &my_dsn);
+	status = trace_append(flags, id, stats, rcpt, relay, tstats, &my_dsn);
 	return (status);
     }
 
@@ -314,13 +332,13 @@ int     bounce_append_intern(int flags, const char *id, MSG_STATS *stats,
 			   SEND_ATTR_INT(MAIL_ATTR_NREQ, BOUNCE_CMD_APPEND),
 				SEND_ATTR_INT(MAIL_ATTR_FLAGS, flags),
 				SEND_ATTR_STR(MAIL_ATTR_QUEUEID, id),
-				SEND_ATTR_FUNC(rcpt_print, (void *) rcpt),
-				SEND_ATTR_FUNC(dsn_print, (void *) &my_dsn),
+			    SEND_ATTR_FUNC(rcpt_print, (const void *) rcpt),
+			  SEND_ATTR_FUNC(dsn_print, (const void *) &my_dsn),
 				ATTR_TYPE_END) == 0
 	    && ((flags & DEL_REQ_FLAG_RECORD) == 0
-		|| trace_append(flags, id, stats, rcpt, relay,
+		|| trace_append(flags, id, stats, rcpt, relay, tstats,
 				&my_dsn) == 0)) {
-	    log_adhoc(id, stats, rcpt, relay, &my_dsn, log_status);
+	    log_adhoc(id, stats, rcpt, relay, tstats, &my_dsn, log_status);
 	    status = (var_soft_bounce ? -1 : 0);
 	} else if ((flags & BOUNCE_FLAG_CLEAN) == 0) {
 	    VSTRING *junk = vstring_alloc(100);
@@ -329,7 +347,8 @@ int     bounce_append_intern(int flags, const char *id, MSG_STATS *stats,
 	    vstring_sprintf(junk, "%s or %s service failure",
 			    var_bounce_service, var_trace_service);
 	    my_dsn.reason = vstring_str(junk);
-	    status = defer_append_intern(flags, id, stats, rcpt, relay, &my_dsn);
+	    status = defer_append_intern(flags, id, stats, rcpt, relay, tstats,
+					 &my_dsn);
 	    vstring_free(junk);
 	} else {
 	    status = -1;
@@ -342,7 +361,7 @@ int     bounce_append_intern(int flags, const char *id, MSG_STATS *stats,
 /* bounce_flush - flush the bounce log and deliver to the sender */
 
 int     bounce_flush(int flags, const char *queue, const char *id,
-		             const char *encoding, int smtputf8,
+		             const char *encoding, int sendopts,
 		             const char *sender, const char *dsn_envid,
 		             int dsn_ret)
 {
@@ -360,7 +379,7 @@ int     bounce_flush(int flags, const char *queue, const char *id,
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUE, queue),
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUEID, id),
 			    SEND_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-			    SEND_ATTR_INT(MAIL_ATTR_SMTPUTF8, smtputf8),
+			    SEND_ATTR_INT(MAIL_ATTR_SENDOPTS, sendopts),
 			    SEND_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			    SEND_ATTR_STR(MAIL_ATTR_DSN_ENVID, dsn_envid),
 			    SEND_ATTR_INT(MAIL_ATTR_DSN_RET, dsn_ret),
@@ -377,7 +396,7 @@ int     bounce_flush(int flags, const char *queue, const char *id,
 /* bounce_flush_verp - verpified notification */
 
 int     bounce_flush_verp(int flags, const char *queue, const char *id,
-			          const char *encoding, int smtputf8,
+			          const char *encoding, int sendopts,
 			          const char *sender, const char *dsn_envid,
 			          int dsn_ret, const char *verp_delims)
 {
@@ -395,7 +414,7 @@ int     bounce_flush_verp(int flags, const char *queue, const char *id,
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUE, queue),
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUEID, id),
 			    SEND_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-			    SEND_ATTR_INT(MAIL_ATTR_SMTPUTF8, smtputf8),
+			    SEND_ATTR_INT(MAIL_ATTR_SENDOPTS, sendopts),
 			    SEND_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			    SEND_ATTR_STR(MAIL_ATTR_DSN_ENVID, dsn_envid),
 			    SEND_ATTR_INT(MAIL_ATTR_DSN_RET, dsn_ret),
@@ -413,10 +432,10 @@ int     bounce_flush_verp(int flags, const char *queue, const char *id,
 /* bounce_one - send notice for one recipient */
 
 int     bounce_one(int flags, const char *queue, const char *id,
-		           const char *encoding, int smtputf8,
+		           const char *encoding, int sendopts,
 		           const char *sender, const char *dsn_envid,
 		           int dsn_ret, MSG_STATS *stats, RECIPIENT *rcpt,
-		           const char *relay, DSN *dsn)
+	               const char *relay, const TLS_STATS *tstats, DSN *dsn)
 {
     DSN     my_dsn = *dsn;
     DSN    *dsn_res;
@@ -435,21 +454,23 @@ int     bounce_one(int flags, const char *queue, const char *id,
     if (delivery_status_filter != 0
     && (dsn_res = dsn_filter_lookup(delivery_status_filter, &my_dsn)) != 0) {
 	if (dsn_res->status[0] == '4')
-	    return (defer_append_intern(flags, id, stats, rcpt, relay, dsn_res));
+	    return (defer_append_intern(flags, id, stats, rcpt, relay, tstats,
+					dsn_res));
 	my_dsn = *dsn_res;
     }
-    return (bounce_one_intern(flags, queue, id, encoding, smtputf8, sender,
-			  dsn_envid, dsn_ret, stats, rcpt, relay, &my_dsn));
+    return (bounce_one_intern(flags, queue, id, encoding, sendopts, sender,
+			      dsn_envid, dsn_ret, stats, rcpt, relay, tstats,
+			      &my_dsn));
 }
 
 /* bounce_one_intern - send notice for one recipient */
 
 int     bounce_one_intern(int flags, const char *queue, const char *id,
-			          const char *encoding, int smtputf8,
+			          const char *encoding, int sendopts,
 			          const char *sender, const char *dsn_envid,
 			          int dsn_ret, MSG_STATS *stats,
 			          RECIPIENT *rcpt, const char *relay,
-			          DSN *dsn)
+			          const TLS_STATS *tstats, DSN *dsn)
 {
     DSN     my_dsn = *dsn;
     int     status;
@@ -460,7 +481,7 @@ int     bounce_one_intern(int flags, const char *queue, const char *id,
      */
     if (flags & DEL_REQ_FLAG_MTA_VRFY) {
 	my_dsn.action = "undeliverable";
-	status = verify_append(id, stats, rcpt, relay, &my_dsn,
+	status = verify_append(id, stats, rcpt, relay, tstats, &my_dsn,
 			       DEL_RCPT_STAT_BOUNCE);
 	return (status);
     }
@@ -471,7 +492,7 @@ int     bounce_one_intern(int flags, const char *queue, const char *id,
      */
     if (flags & DEL_REQ_FLAG_USR_VRFY) {
 	my_dsn.action = "undeliverable";
-	status = trace_append(flags, id, stats, rcpt, relay, &my_dsn);
+	status = trace_append(flags, id, stats, rcpt, relay, tstats, &my_dsn);
 	return (status);
     }
 
@@ -480,7 +501,8 @@ int     bounce_one_intern(int flags, const char *queue, const char *id,
      * based procedure.
      */
     else if (var_soft_bounce) {
-	return (bounce_append_intern(flags, id, stats, rcpt, relay, &my_dsn));
+	return (bounce_append_intern(flags, id, stats, rcpt, relay, tstats,
+				     &my_dsn));
     }
 
     /*
@@ -503,17 +525,17 @@ int     bounce_one_intern(int flags, const char *queue, const char *id,
 				SEND_ATTR_STR(MAIL_ATTR_QUEUE, queue),
 				SEND_ATTR_STR(MAIL_ATTR_QUEUEID, id),
 				SEND_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-				SEND_ATTR_INT(MAIL_ATTR_SMTPUTF8, smtputf8),
+				SEND_ATTR_INT(MAIL_ATTR_SENDOPTS, sendopts),
 				SEND_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			      SEND_ATTR_STR(MAIL_ATTR_DSN_ENVID, dsn_envid),
 				SEND_ATTR_INT(MAIL_ATTR_DSN_RET, dsn_ret),
-				SEND_ATTR_FUNC(rcpt_print, (void *) rcpt),
-				SEND_ATTR_FUNC(dsn_print, (void *) &my_dsn),
+			    SEND_ATTR_FUNC(rcpt_print, (const void *) rcpt),
+			  SEND_ATTR_FUNC(dsn_print, (const void *) &my_dsn),
 				ATTR_TYPE_END) == 0
 	    && ((flags & DEL_REQ_FLAG_RECORD) == 0
-		|| trace_append(flags, id, stats, rcpt, relay,
+		|| trace_append(flags, id, stats, rcpt, relay, tstats,
 				&my_dsn) == 0)) {
-	    log_adhoc(id, stats, rcpt, relay, &my_dsn, "bounced");
+	    log_adhoc(id, stats, rcpt, relay, tstats, &my_dsn, "bounced");
 	    status = 0;
 	} else if ((flags & BOUNCE_FLAG_CLEAN) == 0) {
 	    VSTRING *junk = vstring_alloc(100);
@@ -522,7 +544,8 @@ int     bounce_one_intern(int flags, const char *queue, const char *id,
 	    vstring_sprintf(junk, "%s or %s service failure",
 			    var_bounce_service, var_trace_service);
 	    my_dsn.reason = vstring_str(junk);
-	    status = defer_append_intern(flags, id, stats, rcpt, relay, &my_dsn);
+	    status = defer_append_intern(flags, id, stats, rcpt, relay, tstats,
+					 &my_dsn);
 	    vstring_free(junk);
 	} else {
 	    status = -1;
