@@ -143,7 +143,7 @@ typedef struct SMTP_TLS_POLICY {
 extern void smtp_tls_list_init(void);
 extern int smtp_tls_policy_cache_query(DSN_BUF *, SMTP_TLS_POLICY *, SMTP_ITERATOR *);
 extern void smtp_tls_policy_cache_flush(void);
-extern int  smtp_tls_authorize_mx_hostname(SMTP_TLS_POLICY *, const char *);
+extern int smtp_tls_authorize_mx_hostname(SMTP_TLS_POLICY *, const char *);
 
  /*
   * Macros must use distinct names for local temporary variables, otherwise
@@ -329,7 +329,9 @@ typedef struct SMTP_STATE {
 #define SMTP_FEATURE_REQTLS		(1<<23)	/* RFC 8689 */
 
  /*
-  * Features that passivate under the endpoint.
+  * Features that passivate under the endpoint. Be sure to passivate all
+  * features that are needed in SMTP_KEY_MASK_SCACHE_DEST_LABEL, otherwise a
+  * reused connection may be stored under the wrong key.
   */
 #define SMTP_FEATURE_ENDPOINT_MASK \
 	(~(SMTP_FEATURE_BEST_MX | SMTP_FEATURE_RSET_REJECTED \
@@ -684,6 +686,7 @@ extern void smtp_rcpt_done(SMTP_STATE *, SMTP_RESP *, RECIPIENT *);
 #define SMTP_MISC_FAIL_NONE		0
 #define SMTP_MISC_FAIL_THROTTLE		(1<<0)
 #define SMTP_MISC_FAIL_SOFT_NON_FINAL	(1<<1)
+#define SMTP_MISC_FAIL_DONT_CACHE	(1<<2)
 extern int smtp_sess_fail(SMTP_STATE *);
 extern int PRINTFLIKE(5, 6) smtp_misc_fail(SMTP_STATE *, int, const char *,
 				             SMTP_RESP *, const char *,...);
@@ -729,12 +732,15 @@ char   *smtp_key_prefix(VSTRING *, const char *, SMTP_ITERATOR *, int);
 #define SMTP_KEY_FLAG_ADDR		(1<<5)	/* remote address */
 #define SMTP_KEY_FLAG_PORT		(1<<6)	/* remote port */
 #define SMTP_KEY_FLAG_TLS_LEVEL		(1<<7)	/* requested TLS level */
+#define SMTP_KEY_FLAG_REQ_SMTPUTF8	(1<<8)	/* SMTPUTF8 is required */
+#define SMTP_KEY_FLAG_REQTLS_LEVEL	(1<<9)	/* REQUIRETLS enforcement */
 
 #define SMTP_KEY_MASK_ALL \
 	(SMTP_KEY_FLAG_SERVICE | SMTP_KEY_FLAG_SENDER | \
 	SMTP_KEY_FLAG_REQ_NEXTHOP | \
 	SMTP_KEY_FLAG_CUR_NEXTHOP | SMTP_KEY_FLAG_HOSTNAME | \
-	SMTP_KEY_FLAG_ADDR | SMTP_KEY_FLAG_PORT | SMTP_KEY_FLAG_TLS_LEVEL)
+	SMTP_KEY_FLAG_ADDR | SMTP_KEY_FLAG_PORT | SMTP_KEY_FLAG_TLS_LEVEL | \
+	SMTP_KEY_FLAG_REQ_SMTPUTF8 | SMTP_KEY_FLAG_REQTLS_LEVEL)
 
  /*
   * Conditional lookup-key flags for cached connections that may be
@@ -770,10 +776,15 @@ char   *smtp_key_prefix(VSTRING *, const char *, SMTP_ITERATOR *, int);
   * (REQ_NEXTHOP) prevents false sharing of TLS identities (the destination
   * key links only to appropriate endpoint lookup keys). The SERVICE
   * attribute is a proxy for all request-independent configuration details.
+  * 
+  * Be sure to include all features that are preserved in
+  * SMTP_FEATURE_ENDPOINT_MASK, otherwise a reused connection may be stored
+  * under the wrong key.
   */
 #define SMTP_KEY_MASK_SCACHE_DEST_LABEL \
 	(SMTP_KEY_FLAG_SERVICE | COND_SASL_SMTP_KEY_FLAG_SENDER \
-	| SMTP_KEY_FLAG_REQ_NEXTHOP | SMTP_KEY_FLAG_TLS_LEVEL)
+	| SMTP_KEY_FLAG_REQ_NEXTHOP | SMTP_KEY_FLAG_TLS_LEVEL \
+	| SMTP_KEY_FLAG_REQ_SMTPUTF8 | SMTP_KEY_FLAG_REQTLS_LEVEL)
 
  /*
   * Connection-cache endpoint lookup key. The SENDER, CUR_NEXTHOP, HOSTNAME,
@@ -782,13 +793,18 @@ char   *smtp_key_prefix(VSTRING *, const char *, SMTP_ITERATOR *, int);
   * when different SASL credentials or TLS identities may be required for
   * different deliveries to the same IP address and port. The SERVICE
   * attribute is a proxy for all request-independent configuration details.
+  * 
+  * Be sure to include all features that are preserved in
+  * SMTP_FEATURE_ENDPOINT_MASK, otherwise a reused connection may be stored
+  * under the wrong key.
   */
 #define SMTP_KEY_MASK_SCACHE_ENDP_LABEL \
 	(SMTP_KEY_FLAG_SERVICE | COND_SASL_SMTP_KEY_FLAG_SENDER \
 	| COND_SASL_SMTP_KEY_FLAG_CUR_NEXTHOP \
 	| COND_SASL_SMTP_KEY_FLAG_HOSTNAME \
-	| COND_TLS_SMTP_KEY_FLAG_CUR_NEXTHOP | SMTP_KEY_FLAG_ADDR | \
-	SMTP_KEY_FLAG_PORT | SMTP_KEY_FLAG_TLS_LEVEL)
+	| COND_TLS_SMTP_KEY_FLAG_CUR_NEXTHOP | SMTP_KEY_FLAG_ADDR \
+	| SMTP_KEY_FLAG_PORT | SMTP_KEY_FLAG_TLS_LEVEL \
+	| SMTP_KEY_FLAG_REQ_SMTPUTF8 | SMTP_KEY_FLAG_REQTLS_LEVEL)
 
  /*
   * Silly little macros.
@@ -845,6 +861,24 @@ extern void smtp_tlsrpt_set_ehlo_resp(SMTP_STATE *, const char *ehlo_resp);
 
 #endif					/* USE_TLSRPT && USE_TLS */
 
+ /*
+  * This delivery requires SMTPUTF8 server support if the sender requested
+  * SMTPUTF8 support AND the delivery request involves at least one UTF-8
+  * envelope address or header value.
+  * 
+  * If the sender requested SMTPUTF8 support but the delivery request involves
+  * no UTF-8 envelope address or header value, then we could still deliver
+  * such mail to a non-SMTPUTF8 server, except that we must either
+  * uxtext-encode ORCPT parameters or not send them. We cannot encode the
+  * ORCPT in xtext, because legacy SMTP requires that the unencoded address
+  * consist entirely of printable (graphic and white space) characters from
+  * the US-ASCII repertoire (RFC 3461 section 4). A correct uxtext encoder
+  * will produce a result that an xtext decoder will pass through unchanged.
+  */
+#define DELIVERY_REQUIRES_SMTPUTF8(request) \
+	(((request)->sendopts & SMTPUTF8_FLAG_REQUESTED) \
+	&& ((request)->sendopts & SMTPUTF8_FLAG_DERIVED))
+
 /* LICENSE
 /* .ad
 /* .fi
