@@ -629,33 +629,6 @@ int     smtp_helo(SMTP_STATE *state)
 	msg_info("server features: 0x%x size %.0f",
 		 session->features, (double) session->size_limit);
 
-    /*
-     * Decide if this delivery requires SMTPUTF8 server support.
-     * 
-     * For now, we require that the remote SMTP server supports SMTPUTF8 when
-     * the sender requested SMTPUTF8 support.
-     * 
-     * XXX EAI Refine this to: the sender requested SMTPUTF8 support AND the
-     * delivery request involves at least one UTF-8 envelope address or
-     * header value.
-     * 
-     * If the sender requested SMTPUTF8 support but the delivery request
-     * involves no UTF-8 envelope address or header value, then we could
-     * still deliver such mail to a non-SMTPUTF8 server, except that we must
-     * either uxtext-encode ORCPT parameters or not send them. We cannot
-     * encode the ORCPT in xtext, because legacy SMTP requires that the
-     * unencoded address consist entirely of printable (graphic and white
-     * space) characters from the US-ASCII repertoire (RFC 3461 section 4). A
-     * correct uxtext encoder will produce a result that an xtext decoder
-     * will pass through unchanged.
-     * 
-     * XXX Should we try to encode headers with RFC 2047 when delivering to a
-     * non-SMTPUTF8 server? That could make life easier for mailing lists.
-     */
-#define DELIVERY_REQUIRES_SMTPUTF8 \
-	((request->sendopts & SMTPUTF8_FLAG_REQUESTED) \
-	&& (request->sendopts & SMTPUTF8_FLAG_DERIVED))
-
     /*
      * Require that the server supports SMTPUTF8 when delivery requires
      * SMTPUTF8.
@@ -663,10 +636,14 @@ int     smtp_helo(SMTP_STATE *state)
      * Fix 20140706: moved this before negotiating TLS, AUTH, and so on.
      * 
      * Fix 20250824: try multiple servers before giving up.
+     * 
+     * Fix 20250911: do not cache this session because it does not satisfy the
+     * requirement expressed in the cache storage key.
      */
     if ((session->features & SMTP_FEATURE_SMTPUTF8) == 0
-	&& DELIVERY_REQUIRES_SMTPUTF8)
-	return (smtp_misc_fail(state, SMTP_MISC_FAIL_SOFT_NON_FINAL,
+	&& DELIVERY_REQUIRES_SMTPUTF8(request))
+	return (smtp_misc_fail(state, SMTP_MISC_FAIL_DONT_CACHE
+			       | SMTP_MISC_FAIL_SOFT_NON_FINAL,
 			       DSN_BY_LOCAL_MTA,
 			       SMTP_RESP_FAKE(&fake, "5.6.7"),
 			       "message requires SMTPUTF8, but no "
@@ -706,7 +683,8 @@ int     smtp_helo(SMTP_STATE *state)
 		    smtp_tls_stat_decide_reqtls(state->tls_stats,
 						TLS_STAT_VIOLATION,
 						TLS_STAT_ENF_FULL);
-		return (smtp_misc_fail(state, SMTP_MISC_FAIL_SOFT_NON_FINAL,
+		return (smtp_misc_fail(state, SMTP_MISC_FAIL_DONT_CACHE
+				       | SMTP_MISC_FAIL_SOFT_NON_FINAL,
 				       DSN_BY_LOCAL_MTA,
 				       SMTP_RESP_FAKE(&fake, "5.7.30"),
 				       "REQUIRETLS Failure: sender "
@@ -901,7 +879,8 @@ int     smtp_helo(SMTP_STATE *state)
 						       TLS_STAT_ENF_FULL);
 		/* Then, REQUIRETLS failure must take precedence over other. */
 		if (TLS_REQUIRED_BY_REQTLS_POLICY(state->reqtls_level)) {
-		    return (smtp_misc_fail(state, SMTP_MISC_FAIL_SOFT_NON_FINAL,
+		    return (smtp_misc_fail(state, SMTP_MISC_FAIL_DONT_CACHE
+					   | SMTP_MISC_FAIL_SOFT_NON_FINAL,
 					   DSN_BY_LOCAL_MTA,
 					   SMTP_RESP_FAKE(&fake, "5.7.10"),
 					   "REQUIRETLS Failure: "
@@ -957,7 +936,8 @@ int     smtp_helo(SMTP_STATE *state)
 						       TLS_STAT_ENF_FULL);
 		/* Then, REQUIRETLS failure must take precedence over other. */
 		if (TLS_REQUIRED_BY_REQTLS_POLICY(state->reqtls_level))
-		    return (smtp_misc_fail(state, SMTP_MISC_FAIL_SOFT_NON_FINAL,
+		    return (smtp_misc_fail(state, SMTP_MISC_FAIL_DONT_CACHE
+					   | SMTP_MISC_FAIL_SOFT_NON_FINAL,
 					   DSN_BY_LOCAL_MTA,
 					   SMTP_RESP_FAKE(&fake, "5.7.30"),
 					   "REQUIRETLS Failure: "
@@ -1336,7 +1316,8 @@ static int smtp_start_tls(SMTP_STATE *state)
 		if (state->tls_stats)
 		    smtp_tls_stat_decide_reqtls(state->tls_stats,
 				     TLS_STAT_VIOLATION, TLS_STAT_ENF_FULL);
-		return (smtp_misc_fail(state, SMTP_MISC_FAIL_SOFT_NON_FINAL,
+		return (smtp_misc_fail(state, SMTP_MISC_FAIL_DONT_CACHE
+				       | SMTP_MISC_FAIL_SOFT_NON_FINAL,
 				       DSN_BY_LOCAL_MTA,
 				       SMTP_RESP_FAKE(&fake, "5.7.10"),
 				       "REQUIRETLS Failure: sender "
@@ -1349,7 +1330,7 @@ static int smtp_start_tls(SMTP_STATE *state)
 	    } else if (state->reqtls_level > SMTP_REQTLS_POLICY_ACT_DISABLE) {
 		if (state->tls_stats)
 		    smtp_tls_stat_decide_reqtls(state->tls_stats,
-						TLS_STAT_VIOLATION,
+						TLS_STAT_COMPLIANT,
 						TLS_STAT_ENF_RELAXED);
 	    }
 	    return (smtp_site_fail(state, DSN_BY_LOCAL_MTA,
